@@ -7,7 +7,7 @@ import logging
 from contextlib import contextmanager as ctxmgr
 from datetime import datetime
 from pathlib import Path
-from typing import Awaitable, Callable, Iterator, List
+from typing import Awaitable, Callable, Iterator, List, cast
 from uuid import UUID
 
 import attr
@@ -21,7 +21,7 @@ from ai.backend.storage.exception import ExecutionError
 
 from ..abc import AbstractVolume
 from ..context import Context
-from ..exception import InvalidSubpathError, VFolderNotFoundError
+from ..exception import InvalidSubpathError, StorageProxyError, VFolderNotFoundError
 from ..types import VFolderCreationOptions
 from ..utils import check_params, log_manager_api_entry
 
@@ -162,7 +162,7 @@ async def clone_vfolder(request: web.Request) -> web.Response:
             {
                 t.Key("src_volume"): t.String(),
                 t.Key("src_vfid"): tx.UUID(),
-                t.Key("dst_volume"): t.String(),
+                t.Key("dst_volume"): t.String() | t.Null,
                 t.Key("dst_vfid"): tx.UUID(),
                 t.Key("options", default=None): t.Null | VFolderCreationOptions.as_trafaret(),
             },
@@ -170,13 +170,13 @@ async def clone_vfolder(request: web.Request) -> web.Response:
     ) as params:
         await log_manager_api_entry(log, "clone_vfolder", params)
         ctx: Context = request.app["ctx"]
+        if params["dst_volume"] is not None and params["dst_volume"] != params["src_volume"]:
+            raise StorageProxyError("Cross-volume vfolder cloning is not implemented yet")
         async with ctx.get_volume(params["src_volume"]) as src_volume:
-            async with ctx.get_volume(params["dst_volume"]) as dst_volume:
-                await src_volume.clone_vfolder(
-                    params["src_vfid"],
-                    dst_volume,
-                    params["dst_vfid"],
-                )
+            await src_volume.clone_vfolder(
+                params["src_vfid"],
+                params["dst_vfid"],
+            )
         return web.Response(status=204)
 
 
@@ -242,7 +242,7 @@ async def get_performance_metric(request: web.Request) -> web.Response:
             metric = await volume.get_performance_metric()
             return web.json_response(
                 {
-                    "metric": attr.asdict(metric),
+                    "metric": attr.asdict(cast(attr.AttrsInstance, metric)),
                 },
             )
 
@@ -374,6 +374,33 @@ async def get_vfolder_usage(request: web.Request) -> web.Response:
             )
 
 
+async def get_vfolder_used_bytes(request: web.Request) -> web.Response:
+    async with check_params(
+        request,
+        t.Dict(
+            {
+                t.Key("volume"): t.String(),
+                t.Key("vfid"): tx.UUID(),
+            },
+        ),
+    ) as params:
+        try:
+            await log_manager_api_entry(log, "get_vfolder_used_bytes", params)
+            ctx: Context = request.app["ctx"]
+            async with ctx.get_volume(params["volume"]) as volume:
+                usage = await volume.get_used_bytes(params["vfid"])
+                return web.json_response(
+                    {
+                        "used_bytes": usage,
+                    },
+                )
+        except ExecutionError:
+            return web.Response(
+                status=500,
+                reason="Storage server is busy. Please try again",
+            )
+
+
 async def get_quota(request: web.Request) -> web.Response:
     async with check_params(
         request,
@@ -642,6 +669,7 @@ async def init_manager_app(ctx: Context) -> web.Application:
     app.router.add_route("GET", "/volume/quota", get_quota)
     app.router.add_route("PATCH", "/volume/quota", set_quota)
     app.router.add_route("GET", "/folder/usage", get_vfolder_usage)
+    app.router.add_route("GET", "/folder/used-bytes", get_vfolder_used_bytes)
     app.router.add_route("GET", "/folder/fs-usage", get_vfolder_fs_usage)
     app.router.add_route("POST", "/folder/file/mkdir", mkdir)
     app.router.add_route("POST", "/folder/file/list", list_files)
